@@ -2,7 +2,9 @@
 
 
 class AlbaException(Exception):
-    pass
+    def __init__(self, message, errors=None):
+        super(AlbaException, self).__init__(message)
+        self.errors = errors or {}
 
 
 class UniqueError(AlbaException):
@@ -17,7 +19,6 @@ class MissArgumentError(AlbaException, ValueError):
     pass
 
 
-
 CODE2EXCEPTION = {
     'unknown': AlbaException,
     'common': AlbaException,
