@@ -1,9 +1,12 @@
 # -*- coding: utf-8 -*-
-import urllib
+from __future__ import unicode_literals
+
+from six import text_type
+from six.moves.urllib.parse import urlparse, quote
+
 import base64
 import hashlib
 import hmac
-import urlparse
 
 DEFAULT_SIGN_EXCLUDE = ['check', 'mac', 'check_gate_internal']
 
@@ -12,14 +15,14 @@ def sign(method, url, params, secret_key, exclude=DEFAULT_SIGN_EXCLUDE):
     """
     Типовой метод для подписи HTTP запросов
     """
-    url_parsed = urlparse.urlparse(url)
+    url_parsed = urlparse(url)
     keys = [param for param in params if param not in exclude]
     keys.sort()
 
     result = []
     for key in keys:
-        value = urllib.quote(
-            unicode(params.get(key) or '').encode('utf-8'),
+        value = quote(
+            text_type(params.get(key) or '').encode('utf-8'),
             safe='~'
         )
         result.append('{}={}'.format(key, value))
@@ -30,10 +33,9 @@ def sign(method, url, params, secret_key, exclude=DEFAULT_SIGN_EXCLUDE):
         url_parsed.path,
         "&".join(result)
     ])
-
     digest = hmac.new(
-        secret_key,
-        data,
+        secret_key.encode('utf-8'),
+        data.encode('utf-8'),
         hashlib.sha256
     ).digest()
     signature = base64.b64encode(digest)
